#ifndef DUNE_TECTONIC_MINIMISATION_HH
#define DUNE_TECTONIC_MINIMISATION_HH

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/common/stdstreams.hh>

#include <dune/fufem/arithmetic.hh>
#include <dune/fufem/interval.hh>
#include <dune/tnnmg/problem-classes/bisection.hh>

#include "mydirectionalconvexfunction.hh"

template <class Functional>
double lineSearch(Functional const &J,
                  typename Functional::LocalVector const &x,
                  typename Functional::LocalVector const &v,
                  Bisection const &bisection) {
  MyDirectionalConvexFunction<typename Functional::Nonlinearity> const JRest(
      computeDirectionalA(J.A, v), computeDirectionalb(J.A, J.b, x, v), *J.phi,
      x, v);

  int count;
  return bisection.minimize(JRest, 0.0, 0.0, count);
}

template <class Functional>
void minimise(Functional const &J, typename Functional::LocalVector &x,
              size_t steps, Bisection const &bisection) {
  using LocalVector = typename Functional::LocalVector;
  auto const diff = [](LocalVector const &a, LocalVector const &b) {
    LocalVector tmp = a;
    tmp -= b;
    return tmp.two_norm();
  };

  LocalVector x_initial = x;
  LocalVector x_o = x;

  for (size_t step = 0; step < steps; ++step) {
    LocalVector v;
    J.descentDirection(x, v);

    double const vnorm = v.two_norm();
    if (vnorm <= 0.0)
      return;
    v /= vnorm;

    double const alpha = lineSearch(J, x, v, bisection);
    Arithmetic::addProduct(x, alpha, v);

    if (alpha < 1e-14) // TODO
      break;

    double const correction = diff(x, x_o);
    double const overallCorrection = diff(x, x_initial);
    if (overallCorrection <= 0.0)
      return;

    double const correctionQuotient = correction / overallCorrection;
    if (correctionQuotient < 0.1) // enough descent; TODO
      break;

    x_o = x;
  }
}
#endif
